"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const get_1 = __importDefault(require("./package-shared/actions/get"));
const post_1 = __importDefault(require("./package-shared/actions/post"));
const get_schema_1 = __importDefault(require("./package-shared/actions/get-schema"));
const client_1 = __importDefault(require("./client"));
const sql_generator_1 = __importDefault(require("./package-shared/functions/dsql/sql/sql-generator"));
const sql_insert_generator_1 = __importDefault(require("./package-shared/functions/dsql/sql/sql-insert-generator"));
const sql_delete_generator_1 = __importDefault(require("./package-shared/functions/dsql/sql/sql-delete-generator"));
const trim_sql_1 = __importDefault(require("./package-shared/utils/trim-sql"));
const parseCookies_1 = __importDefault(require("./package-shared/utils/backend/parseCookies"));
const conn_db_handler_1 = __importDefault(require("./package-shared/utils/db/conn-db-handler"));
const encrypt_1 = __importDefault(require("./package-shared/functions/dsql/encrypt"));
const decrypt_1 = __importDefault(require("./package-shared/functions/dsql/decrypt"));
const hashPassword_1 = __importDefault(require("./package-shared/functions/dsql/hashPassword"));
const crud_1 = __importDefault(require("./package-shared/utils/data-fetching/crud"));
const method_crud_1 = __importDefault(require("./package-shared/utils/data-fetching/method-crud"));
const debug_log_1 = __importDefault(require("./package-shared/utils/logging/debug-log"));
const parse_env_1 = __importDefault(require("./package-shared/utils/parse-env"));
const crud_2 = __importDefault(require("./package-shared/api/crud"));
const user_1 = __importDefault(require("./package-shared/api/user"));
const local_user_1 = __importDefault(require("./package-shared/api/user/local-user"));
const media_1 = __importDefault(require("./package-shared/api/media"));
const dbHandler_1 = __importDefault(require("./package-shared/functions/backend/dbHandler"));
/**
 * User Functions Object
 */
// const user = {
//     createUser: createUser,
//     deleteUser,
//     loginUser: loginUser,
//     sendEmailCode: sendEmailCode,
//     logoutUser: logoutUser,
//     userAuth: userAuth,
//     reAuthUser: reAuthUser,
//     updateUser: updateUser,
//     getUser: getUser,
//     getToken: getToken,
//     validateToken: validateToken,
//     validateTempEmailCode,
//     social: {
//         loginWithGoogle: loginWithGoogle,
//         loginWithGithub: loginWithGithub,
//     },
// };
/**
 * API Functions Object
 */
const api = {
    crud: crud_2.default,
    media: media_1.default,
    user: user_1.default,
};
/**
 * SQL Utils
 */
const sql = {
    sqlGenerator: sql_generator_1.default,
    sqlInsertGenerator: sql_insert_generator_1.default,
    sqlDeleteGenerator: sql_delete_generator_1.default,
    trim: trim_sql_1.default,
};
/**
 * Main Export
 */
const datasquirel = {
    /**
     * API Actions
     */
    api,
    /**
     * Get Action
     */
    get: get_1.default,
    /**
     * Post Action
     */
    post: post_1.default,
    user: local_user_1.default,
    getSchema: get_schema_1.default,
    client: client_1.default,
    sql,
    utils: {
        crypto: {
            encrypt: encrypt_1.default,
            decrypt: decrypt_1.default,
            hashPassword: hashPassword_1.default,
        },
        parseCookies: parseCookies_1.default,
        connDbHandler: conn_db_handler_1.default,
        debugLog: debug_log_1.default,
        parseEnv: parse_env_1.default,
    },
    /**
     * Run Crud actions `get`, `insert`, `update`, `delete`
     * @description *Requires global variables `DSQL_USE_LOCAL` and `DSQL_DB_CONN`
     */
    crud: crud_1.default,
    /**
     * Run Crud based on request Methods `GET`, `POST`, `PUT`, `PATCH`
     * @description *Requires global variables `DSQL_USE_LOCAL` and `DSQL_DB_CONN`
     */
    methodCrud: method_crud_1.default,
    /**
     * General Database Handler
     */
    dbHandler: dbHandler_1.default,
};
exports.default = datasquirel;
