"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = get;
const grab_host_names_1 = __importDefault(require("../utils/grab-host-names"));
const serialize_query_1 = __importDefault(require("../utils/serialize-query"));
const debug_log_1 = __importDefault(require("../utils/logging/debug-log"));
const crud_1 = __importDefault(require("../utils/data-fetching/crud"));
const grab_api_key_1 = __importDefault(require("../utils/grab-api-key"));
/**
 * # Make a get request to Datasquirel API
 */
function get(_a) {
    return __awaiter(this, arguments, void 0, function* ({ key, database, query, table, debug, useLocal, apiVersion = "v1", }) {
        const grabedHostNames = (0, grab_host_names_1.default)();
        const { host, port, scheme } = grabedHostNames;
        function debugFn(log, label) {
            (0, debug_log_1.default)({ log, addTime: true, title: "apiGet", label });
        }
        /**
         * Check for local DB settings
         *
         * @description Look for local db settings in `.env` file and by pass the http request if available
         */
        const { DSQL_DB_NAME } = process.env;
        if (useLocal) {
            const result = yield (0, crud_1.default)({
                action: "get",
                table: table || "",
                dbFullName: database,
                query,
            });
            return result;
        }
        /**
         * Make https request
         *
         * @description make a request to datasquirel.com
         */
        const httpResponse = yield new Promise((resolve, reject) => {
            const queryString = (0, serialize_query_1.default)(query);
            if (debug) {
                debugFn(queryString, "queryString");
            }
            let path = `/api/${apiVersion}/crud/${database}/${table}`;
            if (debug) {
                debugFn(path, "path");
            }
            const requestObject = {
                method: "GET",
                headers: {
                    "Content-Type": "application/json",
                    Authorization: (0, grab_api_key_1.default)(key),
                },
                port,
                hostname: host,
                path,
            };
            scheme
                .request(requestObject, 
            /**
             * Callback Function
             *
             * @description https request callback
             */
            (response) => {
                var str = "";
                response.on("data", function (chunk) {
                    str += chunk;
                });
                response.on("end", function () {
                    try {
                        resolve(JSON.parse(str));
                    }
                    catch (error) {
                        reject({
                            error: error.message,
                            result: str,
                        });
                    }
                });
                response.on("error", (err) => {
                    console.log("DSQL get Error,", err.message);
                    resolve(null);
                });
            })
                .end();
        });
        return httpResponse;
    });
}
