"use strict";
// @ts-check
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = post;
const grab_host_names_1 = __importDefault(require("../utils/grab-host-names"));
const post_1 = __importDefault(require("../functions/api/query/post"));
const debug_log_1 = __importDefault(require("../utils/logging/debug-log"));
/**
 * # Make a post request to Datasquirel API
 */
function post(_a) {
    return __awaiter(this, arguments, void 0, function* ({ key, query, queryValues, database, tableName, user_id, useLocal, debug, }) {
        const grabedHostNames = (0, grab_host_names_1.default)();
        const { host, port, scheme } = grabedHostNames;
        if (debug) {
            (0, debug_log_1.default)({
                log: grabedHostNames,
                addTime: true,
                label: "grabedHostNames",
            });
        }
        /**
         * Check for local DB settings
         *
         * @description Look for local db settings in `.env` file and by pass the http request if available
         */
        const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME } = process.env;
        if (useLocal) {
            if (debug) {
                (0, debug_log_1.default)({
                    log: "Using Local DB ...",
                    addTime: true,
                });
            }
            return yield (0, post_1.default)({
                dbFullName: database || "",
                query,
                queryValues,
                tableName,
                debug,
            });
        }
        /**
         * Make https request
         *
         * @description make a request to datasquirel.com
         */
        const httpResponse = yield new Promise((resolve, reject) => {
            const reqPayloadString = JSON.stringify({
                query,
                queryValues,
                database: process.env.DSQL_API_DB_NAME || database,
                tableName: tableName ? tableName : null,
            }).replace(/\n|\r|\n\r/gm, "");
            try {
                JSON.parse(reqPayloadString);
            }
            catch (error) {
                console.log("Error Parsing HTTP response for post action: ", error.message);
                return {
                    success: false,
                    payload: null,
                    error: "Query object is invalid. Please Check query data values",
                };
            }
            const reqPayload = reqPayloadString;
            const requPath = `/api/query/${user_id || grabedHostNames.user_id}/post`;
            const httpsRequest = scheme.request({
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    "Content-Length": Buffer.from(reqPayload).length,
                    Authorization: key ||
                        process.env.DSQL_FULL_ACCESS_API_KEY ||
                        process.env.DSQL_API_KEY,
                },
                port,
                hostname: host,
                path: requPath,
            }, 
            /**
             * Callback Function
             *
             * @description https request callback
             */
            (response) => {
                var str = "";
                response.on("data", function (chunk) {
                    str += chunk;
                });
                response.on("end", function () {
                    try {
                        resolve(JSON.parse(str));
                    }
                    catch (error) {
                        console.log("Route ERROR:", error.message);
                        resolve({
                            success: false,
                            payload: null,
                            error: error.message,
                            errPayload: str,
                        });
                    }
                });
                response.on("error", (err) => {
                    resolve({
                        success: false,
                        payload: null,
                        error: err.message,
                    });
                });
            });
            httpsRequest.write(reqPayload);
            httpsRequest.on("error", (error) => {
                console.log("HTTPS request ERROR =>", error);
            });
            httpsRequest.end();
        });
        return httpResponse;
    });
}
