"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = getUser;
const grab_host_names_1 = __importDefault(require("../../utils/grab-host-names"));
const api_get_user_1 = __importDefault(require("../../functions/api/users/api-get-user"));
const grab_api_path_1 = __importDefault(require("../../utils/backend/users/grab-api-path"));
/**
 * # Get User
 */
function getUser(_a) {
    return __awaiter(this, arguments, void 0, function* ({ apiKey, userId, database, fields, useLocal, apiVersion = "v1", dbUserId, selectAll, }) {
        /**
         * Initialize
         */
        const defaultFields = [
            "id",
            "first_name",
            "last_name",
            "email",
            "username",
            "image",
            "image_thumbnail",
            "verification_status",
            "date_created",
            "date_created_code",
            "date_created_timestamp",
            "date_updated",
            "date_updated_code",
            "date_updated_timestamp",
        ];
        const updatedFields = fields && fields[0] ? [...defaultFields, ...fields] : defaultFields;
        const grabedHostNames = (0, grab_host_names_1.default)();
        const { host, port, scheme } = grabedHostNames;
        const getUserParams = {
            userId,
            fields: [...new Set(updatedFields)],
            database,
            dbUserId,
            selectAll,
        };
        if (useLocal) {
            return yield (0, api_get_user_1.default)(getUserParams);
        }
        /**
         * Make https request
         *
         * @description make a request to datasquirel.com
         */
        const httpResponse = yield new Promise((resolve, reject) => {
            const reqPayload = JSON.stringify(getUserParams);
            const finalAPIKey = apiKey ||
                process.env.DSQL_API_KEY ||
                process.env.DSQL_FULL_ACCESS_API_KEY ||
                process.env.DSQL_READ_ONLY_API_KEY;
            const httpsRequest = scheme.request({
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    "Content-Length": Buffer.from(reqPayload).length,
                    Authorization: finalAPIKey,
                },
                port,
                hostname: host,
                path: (0, grab_api_path_1.default)({
                    paradigm: "auth",
                    action: "get",
                    database,
                    apiVersion,
                    userID: userId,
                }),
            }, 
            /**
             * Callback Function
             *
             * @description https request callback
             */
            (response) => {
                var str = "";
                response.on("data", function (chunk) {
                    str += chunk;
                });
                response.on("end", function () {
                    resolve(JSON.parse(str));
                });
                response.on("error", (err) => {
                    reject(err);
                });
            });
            httpsRequest.write(reqPayload);
            httpsRequest.end();
        });
        return httpResponse;
    });
}
