"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = githubAuth;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const encrypt_1 = __importDefault(require("../../../functions/dsql/encrypt"));
const grab_host_names_1 = __importDefault(require("../../../utils/grab-host-names"));
const api_github_login_1 = __importDefault(require("../../../functions/api/users/social/api-github-login"));
const grab_cookie_expirt_date_1 = __importDefault(require("../../../utils/grab-cookie-expirt-date"));
/**
 * # SERVER FUNCTION: Login with google Function
 */
function githubAuth(_a) {
    return __awaiter(this, arguments, void 0, function* ({ key, code, email, database, clientId, clientSecret, response, encryptionKey, encryptionSalt, additionalFields, user_id, additionalData, secureCookie, }) {
        /**
         * Check inputs
         *
         * @description Check inputs
         */
        const grabedHostNames = (0, grab_host_names_1.default)();
        const { host, port, scheme } = grabedHostNames;
        const COOKIE_EXPIRY_DATE = (0, grab_cookie_expirt_date_1.default)();
        if (!code || (code === null || code === void 0 ? void 0 : code.match(/ /))) {
            return {
                success: false,
                user: null,
                msg: "Please enter Github Access Token",
            };
        }
        if (!database || (database === null || database === void 0 ? void 0 : database.match(/ /))) {
            return {
                success: false,
                user: null,
                msg: "Please provide database slug name you want to access",
            };
        }
        if (!clientId || (clientId === null || clientId === void 0 ? void 0 : clientId.match(/ /))) {
            return {
                success: false,
                user: null,
                msg: "Please enter Github OAUTH client ID",
            };
        }
        /**
         * Initialize HTTP response variable
         */
        let httpResponse;
        /**
         * Check for local DB settings
         *
         * @description Look for local db settings in `.env` file and by pass the http request if available
         */
        const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME, DSQL_KEY, DSQL_REF_DB_NAME, DSQL_FULL_SYNC, } = process.env;
        if ((DSQL_DB_HOST === null || DSQL_DB_HOST === void 0 ? void 0 : DSQL_DB_HOST.match(/./)) &&
            (DSQL_DB_USERNAME === null || DSQL_DB_USERNAME === void 0 ? void 0 : DSQL_DB_USERNAME.match(/./)) &&
            (DSQL_DB_PASSWORD === null || DSQL_DB_PASSWORD === void 0 ? void 0 : DSQL_DB_PASSWORD.match(/./)) &&
            (DSQL_DB_NAME === null || DSQL_DB_NAME === void 0 ? void 0 : DSQL_DB_NAME.match(/./))) {
            /** @type {import("../../../types").DSQL_DatabaseSchemaType | undefined | undefined} */
            let dbSchema;
            try {
                const localDbSchemaPath = path_1.default.resolve(process.cwd(), "dsql.schema.json");
                dbSchema = JSON.parse(fs_1.default.readFileSync(localDbSchemaPath, "utf8"));
            }
            catch (error) { }
            httpResponse = yield (0, api_github_login_1.default)({
                code,
                email: email || undefined,
                clientId,
                clientSecret,
                additionalFields,
                database: DSQL_DB_NAME,
                additionalData,
            });
        }
        else {
            /**
             * Make https request
             *
             * @description make a request to datasquirel.com
             * @type {FunctionReturn} - Https response object
             */
            httpResponse = (yield new Promise((resolve, reject) => {
                const reqPayload = JSON.stringify({
                    code,
                    email,
                    clientId,
                    clientSecret,
                    database,
                    additionalFields,
                    additionalData,
                });
                const httpsRequest = scheme.request({
                    method: "POST",
                    headers: {
                        "Content-Type": "application/json",
                        "Content-Length": Buffer.from(reqPayload).length,
                        Authorization: key ||
                            process.env.DSQL_FULL_ACCESS_API_KEY ||
                            process.env.DSQL_API_KEY,
                    },
                    port,
                    hostname: host,
                    path: `/api/user/${user_id || grabedHostNames.user_id}/github-login`,
                }, 
                /**
                 * Callback Function
                 *
                 * @description https request callback
                 */
                (response) => {
                    var str = "";
                    response.on("data", function (chunk) {
                        str += chunk;
                    });
                    response.on("end", function () {
                        try {
                            resolve(JSON.parse(str));
                        }
                        catch (error) {
                            console.log("Github Auth Error:", error.message);
                            resolve({
                                success: false,
                                user: null,
                                msg: "Something went wrong",
                            });
                        }
                    });
                    response.on("error", (err) => {
                        reject(err);
                    });
                });
                httpsRequest.write(reqPayload);
                httpsRequest.end();
            }));
        }
        ////////////////////////////////////////
        ////////////////////////////////////////
        ////////////////////////////////////////
        /**
         * Make https request
         *
         * @description make a request to datasquirel.com
         */
        if ((httpResponse === null || httpResponse === void 0 ? void 0 : httpResponse.success) && (httpResponse === null || httpResponse === void 0 ? void 0 : httpResponse.user)) {
            let encryptedPayload = (0, encrypt_1.default)({
                data: JSON.stringify(httpResponse.user),
                encryptionKey,
                encryptionSalt,
            });
            const { user, dsqlUserId } = httpResponse;
            const authKeyName = `datasquirel_${dsqlUserId}_${database}_auth_key`;
            const csrfName = `datasquirel_${dsqlUserId}_${database}_csrf`;
            response.setHeader("Set-Cookie", [
                `${authKeyName}=${encryptedPayload};samesite=strict;path=/;HttpOnly=true;Expires=${COOKIE_EXPIRY_DATE}${secureCookie ? ";Secure=true" : ""}`,
                `${csrfName}=${user.csrf_k};samesite=strict;path=/;HttpOnly=true;Expires=${COOKIE_EXPIRY_DATE}`,
            ]);
        }
        return httpResponse;
    });
}
