"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = apiPost;
const lodash_1 = __importDefault(require("lodash"));
const serverError_1 = __importDefault(require("../../backend/serverError"));
const runQuery_1 = __importDefault(require("../../backend/db/runQuery"));
const debug_log_1 = __importDefault(require("../../../utils/logging/debug-log"));
/**
 * # Post Function For API
 */
function apiPost(_a) {
    return __awaiter(this, arguments, void 0, function* ({ query, dbFullName, queryValues, tableName, dbSchema, dbContext, debug, }) {
        var _b;
        if (typeof query === "string" && (query === null || query === void 0 ? void 0 : query.match(/^create |^alter |^drop /i))) {
            return { success: false, msg: "Wrong Input" };
        }
        if (typeof query === "object" &&
            ((_b = query === null || query === void 0 ? void 0 : query.action) === null || _b === void 0 ? void 0 : _b.match(/^create |^alter |^drop /i))) {
            return { success: false, msg: "Wrong Input" };
        }
        let results;
        /**
         * Create new user folder and file
         *
         * @description Create new user folder and file
         */
        try {
            let { result, error } = yield (0, runQuery_1.default)({
                dbFullName,
                query,
                dbSchema,
                queryValuesArray: queryValues,
                tableName,
                dbContext,
                debug,
            });
            if (debug) {
                (0, debug_log_1.default)({
                    log: result,
                    addTime: true,
                    label: "result",
                });
                (0, debug_log_1.default)({
                    log: query,
                    addTime: true,
                    label: "query",
                });
            }
            results = result;
            if (error)
                throw new Error(error);
            let tableSchema;
            if (dbSchema) {
                const targetTable = dbSchema.tables.find((table) => table.tableName === tableName);
                if (targetTable) {
                    const clonedTargetTable = lodash_1.default.cloneDeep(targetTable);
                    delete clonedTargetTable.childTable;
                    delete clonedTargetTable.childrenTables;
                    delete clonedTargetTable.updateData;
                    delete clonedTargetTable.indexes;
                    tableSchema = clonedTargetTable;
                }
            }
            return {
                success: true,
                payload: results,
                error: error,
                schema: tableName && tableSchema ? tableSchema : undefined,
            };
        }
        catch (error) {
            (0, serverError_1.default)({
                component: "/api/query/post/lines-132-142",
                message: error.message,
            });
            return {
                success: false,
                payload: results,
                error: error.message,
            };
        }
    });
}
