"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = githubLogin;
const dbHandler_1 = __importDefault(require("../../backend/dbHandler"));
const httpsRequest_1 = __importDefault(require("../../backend/httpsRequest"));
/**
 * # Login/signup a github user
 */
function githubLogin(_a) {
    return __awaiter(this, arguments, void 0, function* ({ code, clientId, clientSecret, }) {
        let gitHubUser;
        try {
            const response = yield (0, httpsRequest_1.default)({
                method: "POST",
                hostname: "github.com",
                path: `/login/oauth/access_token?client_id=${clientId}&client_secret=${clientSecret}&code=${code}`,
                headers: {
                    Accept: "application/json",
                    "User-Agent": "*",
                },
                scheme: "https",
            });
            const accessTokenObject = JSON.parse(response);
            if (!(accessTokenObject === null || accessTokenObject === void 0 ? void 0 : accessTokenObject.access_token)) {
                return gitHubUser;
            }
            const userDataResponse = yield (0, httpsRequest_1.default)({
                method: "GET",
                hostname: "api.github.com",
                path: "/user",
                headers: {
                    Authorization: `Bearer ${accessTokenObject.access_token}`,
                    "User-Agent": "*",
                },
                scheme: "https",
            });
            gitHubUser = JSON.parse(userDataResponse);
            if (!(gitHubUser === null || gitHubUser === void 0 ? void 0 : gitHubUser.email) && gitHubUser) {
                const existingGithubUser = (yield (0, dbHandler_1.default)({
                    query: `SELECT email FROM users WHERE social_login='1' AND social_platform='github' AND social_id='${gitHubUser.id}'`,
                    values: [gitHubUser.id],
                }));
                if (existingGithubUser && existingGithubUser[0]) {
                    gitHubUser.email = existingGithubUser[0].email;
                }
            }
        }
        catch ( /** @type {any} */error) {
            console.log("ERROR in githubLogin.ts backend function =>", error.message);
        }
        return gitHubUser;
    });
}
