"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = handleSocialDb;
const fs_1 = __importDefault(require("fs"));
const handleNodemailer_1 = __importDefault(require("../../backend/handleNodemailer"));
const path_1 = __importDefault(require("path"));
const addMariadbUser_1 = __importDefault(require("../../backend/addMariadbUser"));
const dbHandler_1 = __importDefault(require("../../backend/dbHandler"));
const encrypt_1 = __importDefault(require("../../dsql/encrypt"));
const addDbEntry_1 = __importDefault(require("../../backend/db/addDbEntry"));
const loginSocialUser_1 = __importDefault(require("./loginSocialUser"));
const grab_dir_names_1 = __importDefault(require("../../../utils/backend/names/grab-dir-names"));
const grab_db_full_name_1 = __importDefault(require("../../../utils/grab-db-full-name"));
/**
 * # Handle Social DB
 */
function handleSocialDb(_a) {
    return __awaiter(this, arguments, void 0, function* ({ database, email, social_platform, payload, invitation, supEmail, additionalFields, debug, loginOnly, apiUserId, }) {
        var _b;
        try {
            const finalDbName = (0, grab_db_full_name_1.default)({
                dbName: database,
                userId: apiUserId,
            });
            const existingSocialUserQUery = `SELECT * FROM users WHERE email = ? AND social_login='1' AND social_platform = ? `;
            const existingSocialUserValues = [email, social_platform];
            if (debug) {
                console.log("handleSocialDb:existingSocialUserQUery", existingSocialUserQUery);
                console.log("handleSocialDb:existingSocialUserValues", existingSocialUserValues);
            }
            let existingSocialUser = (yield (0, dbHandler_1.default)({
                database: finalDbName,
                query: existingSocialUserQUery,
                values: existingSocialUserValues,
            }));
            if (debug) {
                console.log("handleSocialDb:existingSocialUser", existingSocialUser);
            }
            if (existingSocialUser === null || existingSocialUser === void 0 ? void 0 : existingSocialUser[0]) {
                return yield (0, loginSocialUser_1.default)({
                    user: existingSocialUser[0],
                    social_platform,
                    invitation,
                    database: finalDbName,
                    additionalFields,
                    debug,
                });
            }
            else if (loginOnly) {
                return {
                    success: false,
                    payload: null,
                    msg: "User Does not Exist",
                };
            }
            const finalEmail = email ? email : supEmail ? supEmail : null;
            if (!finalEmail) {
                return {
                    success: false,
                    payload: null,
                    msg: "No Email Present",
                };
            }
            const existingEmailOnlyQuery = `SELECT * FROM users WHERE email='${finalEmail}'`;
            if (debug) {
                console.log("handleSocialDb:existingEmailOnlyQuery", existingEmailOnlyQuery);
            }
            let existingEmailOnly = (yield (0, dbHandler_1.default)({
                database: finalDbName,
                query: existingEmailOnlyQuery,
            }));
            if (debug) {
                console.log("handleSocialDb:existingEmailOnly", existingEmailOnly);
            }
            if (existingEmailOnly === null || existingEmailOnly === void 0 ? void 0 : existingEmailOnly[0]) {
                return yield (0, loginSocialUser_1.default)({
                    user: existingEmailOnly[0],
                    social_platform,
                    invitation,
                    database: finalDbName,
                    additionalFields,
                    debug,
                });
            }
            else if (loginOnly) {
                return {
                    success: false,
                    payload: null,
                    msg: "Social Account Creation Not allowed",
                };
            }
            const socialHashedPassword = (0, encrypt_1.default)({
                data: email,
            });
            const data = {
                social_login: "1",
                verification_status: supEmail ? "0" : "1",
                password: socialHashedPassword,
            };
            Object.keys(payload).forEach((key) => {
                data[key] = payload[key];
            });
            const newUser = yield (0, addDbEntry_1.default)({
                dbContext: finalDbName ? "Dsql User" : undefined,
                paradigm: finalDbName ? "Full Access" : undefined,
                dbFullName: finalDbName,
                tableName: "users",
                duplicateColumnName: "email",
                duplicateColumnValue: finalEmail,
                data: Object.assign(Object.assign({}, data), { email: finalEmail }),
            });
            if ((_b = newUser === null || newUser === void 0 ? void 0 : newUser.payload) === null || _b === void 0 ? void 0 : _b.insertId) {
                if (!database) {
                    /**
                     * Add a Mariadb User for this User
                     */
                    yield (0, addMariadbUser_1.default)({ userId: newUser.payload.insertId });
                }
                const newUserQueriedQuery = `SELECT * FROM users WHERE id='${newUser.payload.insertId}'`;
                const newUserQueried = (yield (0, dbHandler_1.default)({
                    database: finalDbName,
                    query: newUserQueriedQuery,
                }));
                if (!newUserQueried || !newUserQueried[0])
                    return {
                        success: false,
                        payload: null,
                        msg: "User Insertion Failed!",
                    };
                if (supEmail && (database === null || database === void 0 ? void 0 : database.match(/^datasquirel$/))) {
                    /**
                     * Send email Verification
                     *
                     * @description Send verification email to newly created agent
                     */
                    let generatedToken = (0, encrypt_1.default)({
                        data: JSON.stringify({
                            id: newUser.payload.insertId,
                            email: supEmail,
                            dateCode: Date.now(),
                        }),
                    });
                    (0, handleNodemailer_1.default)({
                        to: supEmail,
                        subject: "Verify Email Address",
                        text: "Please click the link to verify your email address",
                        html: fs_1.default
                            .readFileSync("./email/send-email-verification-link.html", "utf8")
                            .replace(/{{host}}/, process.env.DSQL_HOST || "")
                            .replace(/{{token}}/, generatedToken || ""),
                    }).then(() => { });
                }
                const { STATIC_ROOT } = (0, grab_dir_names_1.default)();
                if (!STATIC_ROOT) {
                    console.log("Static File ENV not Found!");
                    return {
                        success: false,
                        payload: null,
                        msg: "Static File ENV not Found!",
                    };
                }
                /**
                 * Create new user folder and file
                 *
                 * @description Create new user folder and file
                 */
                if (!database || (database === null || database === void 0 ? void 0 : database.match(/^datasquirel$/))) {
                    let newUserSchemaFolderPath = `${process.env.DSQL_USER_DB_SCHEMA_PATH}/user-${newUser.payload.insertId}`;
                    let newUserMediaFolderPath = path_1.default.join(STATIC_ROOT, `images/user-images/user-${newUser.payload.insertId}`);
                    fs_1.default.mkdirSync(newUserSchemaFolderPath);
                    fs_1.default.mkdirSync(newUserMediaFolderPath);
                    fs_1.default.writeFileSync(`${newUserSchemaFolderPath}/main.json`, JSON.stringify([]), "utf8");
                }
                return yield (0, loginSocialUser_1.default)({
                    user: newUserQueried[0],
                    social_platform,
                    invitation,
                    database: finalDbName,
                    additionalFields,
                    debug,
                });
            }
            else {
                console.log("Social User Failed to insert in 'handleSocialDb.ts' backend function =>", newUser);
                return {
                    success: false,
                    payload: null,
                    msg: "Social User Failed to insert in 'handleSocialDb.ts' backend function",
                };
            }
        }
        catch (error) {
            console.log("ERROR in 'handleSocialDb.ts' backend function =>", error.message);
            return {
                success: false,
                payload: null,
                msg: error.message,
            };
        }
    });
}
