"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = addUsersTableToDb;
const serverError_1 = __importDefault(require("./serverError"));
const createDbFromSchema_1 = __importDefault(require("../../shell/createDbFromSchema"));
const grabNewUsersTableSchema_1 = __importDefault(require("./grabNewUsersTableSchema"));
const grab_required_database_schemas_1 = require("../../shell/createDbFromSchema/grab-required-database-schemas");
/**
 * # Add User Table to Database
 */
function addUsersTableToDb(_a) {
    return __awaiter(this, arguments, void 0, function* ({ userId, database, payload, dbId, }) {
        try {
            const userPreset = (0, grabNewUsersTableSchema_1.default)({ payload });
            if (!userPreset)
                throw new Error("Couldn't Get User Preset!");
            let targetDatabase = (0, grab_required_database_schemas_1.grabPrimaryRequiredDbSchema)({
                dbId,
                userId,
            });
            if (!targetDatabase) {
                throw new Error("Couldn't Find Target Database!");
            }
            let existingTableIndex = targetDatabase === null || targetDatabase === void 0 ? void 0 : targetDatabase.tables.findIndex((table) => table.tableName === "users");
            if (typeof existingTableIndex == "number" && existingTableIndex > 0) {
                targetDatabase.tables[existingTableIndex] = userPreset;
            }
            else {
                targetDatabase.tables.push(userPreset);
            }
            (0, grab_required_database_schemas_1.writeUpdatedDbSchema)({ dbSchema: targetDatabase, userId });
            const dbShellUpdate = yield (0, createDbFromSchema_1.default)({
                userId,
                dbId,
            });
            return dbShellUpdate;
        }
        catch (error) {
            console.log(`addUsersTableToDb.ts ERROR: ${error.message}`);
            (0, serverError_1.default)({
                component: "addUsersTableToDb",
                message: error.message,
                user: { id: userId },
            });
            return false;
        }
    });
}
