"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = dbHandler;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const grab_dsql_connection_1 = __importDefault(require("../../utils/grab-dsql-connection"));
/**
 * # Main DB Handler Function
 * @requires DSQL_DB_CONN - Gobal Variable for Datasquirel Database
 */
function dbHandler(_a) {
    return __awaiter(this, arguments, void 0, function* ({ query, values, noErrorLogs, database, config, }) {
        let CONNECTION;
        let results;
        try {
            CONNECTION = yield (0, grab_dsql_connection_1.default)({ database, config });
            if (query && values) {
                const queryResults = yield CONNECTION.query(query, values);
                results = queryResults[0];
            }
            else {
                const queryResults = yield CONNECTION.query(query);
                results = queryResults[0];
            }
        }
        catch (error) {
            console.log("Connection Info =>", CONNECTION === null || CONNECTION === void 0 ? void 0 : CONNECTION.info);
            if (error.message &&
                typeof error.message == "string" &&
                error.message.match(/Access denied for user.*password/i)) {
                throw new Error("Authentication Failed!");
            }
            if (!noErrorLogs) {
                console.log("ERROR in dbHandler =>", error.message);
                console.log(error);
                const tmpFolder = path_1.default.resolve(process.cwd(), "./.tmp");
                if (!fs_1.default.existsSync(tmpFolder))
                    fs_1.default.mkdirSync(tmpFolder, { recursive: true });
                fs_1.default.appendFileSync(path_1.default.resolve(tmpFolder, "./dbErrorLogs.txt"), JSON.stringify(error, null, 4) + "\n" + Date() + "\n\n\n", "utf8");
            }
            results = null;
        }
        finally {
            yield (CONNECTION === null || CONNECTION === void 0 ? void 0 : CONNECTION.end());
        }
        if (results) {
            return results;
        }
        else {
            return null;
        }
    });
}
