"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = handleNodemailer;
const fs_1 = __importDefault(require("fs"));
const lodash_1 = __importDefault(require("lodash"));
const nodemailer_1 = __importDefault(require("nodemailer"));
/**
 * # Handle mails With Nodemailer
 */
function handleNodemailer(params) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a, _b, _c;
        if (!process.env.DSQL_MAIL_HOST ||
            !process.env.DSQL_MAIL_EMAIL ||
            !process.env.DSQL_MAIL_PASSWORD) {
            return undefined;
        }
        let transporter = nodemailer_1.default.createTransport(Object.assign(Object.assign({ host: process.env.DSQL_MAIL_HOST, port: 465, secure: true }, params.options), { auth: Object.assign({ user: process.env.DSQL_MAIL_EMAIL, pass: process.env.DSQL_MAIL_PASSWORD }, (_a = params.options) === null || _a === void 0 ? void 0 : _a.auth) }));
        if (params.verifyOnly) {
            const isMailServerValid = yield transporter.verify();
            if (!isMailServerValid) {
                return false;
            }
            return true;
        }
        const sender = (() => {
            var _a;
            if ((_a = params.alias) === null || _a === void 0 ? void 0 : _a.match(/support/i))
                return process.env.DSQL_MAIL_EMAIL;
            return process.env.DSQL_MAIL_EMAIL;
        })();
        const mailRootPath = process.env.DSQL_MAIL_ROOT || "./email/index.html";
        let mailRoot = fs_1.default.existsSync(mailRootPath)
            ? fs_1.default.readFileSync(mailRootPath, "utf8")
            : undefined;
        let finalHtml = mailRoot
            ? mailRoot
                .replace(/{{email_body}}/, ((_b = params.html) === null || _b === void 0 ? void 0 : _b.toString()) || "")
                .replace(/{{issue_date}}/, Date().substring(0, 24))
            : (_c = params.html) === null || _c === void 0 ? void 0 : _c.toString();
        try {
            let mailObject = {};
            mailObject["from"] = `"${params.senderName || "Datasquirel"}" <${sender}>`;
            mailObject["sender"] = sender;
            if (params.alias)
                mailObject["replyTo"] = sender;
            mailObject["to"] = params.to;
            mailObject["subject"] = params.subject;
            mailObject["text"] = params.text;
            mailObject["html"] = finalHtml;
            let info = yield transporter.sendMail(Object.assign(Object.assign({}, lodash_1.default.omit(mailObject, ["alias", "senderName", "options"])), mailObject));
            return info;
        }
        catch (error) {
            console.log("ERROR in handleNodemailer Function =>", error.message);
        }
        return undefined;
    });
}
