"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = generateTypeDefinition;
const default_fields_regexp_1 = __importDefault(require("./default-fields-regexp"));
function generateTypeDefinition({ paradigm, table, query, typeDefName, allValuesOptional, addExport, dbName, }) {
    let typeDefinition = ``;
    try {
        const tdName = typeDefName
            ? typeDefName
            : dbName
                ? `DSQL_${dbName}_${table.tableName}`.toUpperCase()
                : `DSQL_${query.single}_${query.single_table}`.toUpperCase();
        const fields = table.fields;
        function typeMap(schemaType) {
            var _a, _b;
            if (schemaType.options && schemaType.options.length > 0) {
                return schemaType.options
                    .map((opt) => {
                    var _a;
                    return ((_a = schemaType.dataType) === null || _a === void 0 ? void 0 : _a.match(/int/i)) ||
                        typeof opt == "number"
                        ? `${opt}`
                        : `"${opt}"`;
                })
                    .join(" | ");
            }
            if ((_a = schemaType.dataType) === null || _a === void 0 ? void 0 : _a.match(/int/i)) {
                return "number";
            }
            if ((_b = schemaType.dataType) === null || _b === void 0 ? void 0 : _b.match(/text|varchar|timestamp/i)) {
                return "string";
            }
            return "string";
        }
        const typesArrayTypeScript = [];
        const typesArrayJavascript = [];
        typesArrayTypeScript.push(`${addExport ? "export " : ""}type ${tdName} = {`);
        typesArrayJavascript.push(`/**\n * @typedef {object} ${tdName}`);
        fields.forEach((field) => {
            var _a;
            const nullValue = allValuesOptional
                ? "?"
                : ((_a = field.fieldName) === null || _a === void 0 ? void 0 : _a.match(default_fields_regexp_1.default))
                    ? "?"
                    : field.notNullValue
                        ? ""
                        : "?";
            typesArrayTypeScript.push(`    ${field.fieldName}${nullValue}: ${typeMap(field)};`);
            typesArrayJavascript.push(` * @property {${typeMap(field)}${nullValue}} ${field.fieldName}`);
        });
        typesArrayTypeScript.push(`}`);
        typesArrayJavascript.push(` */`);
        if (paradigm === null || paradigm === void 0 ? void 0 : paradigm.match(/javascript/i)) {
            typeDefinition = typesArrayJavascript.join("\n");
        }
        if (paradigm === null || paradigm === void 0 ? void 0 : paradigm.match(/typescript/i)) {
            typeDefinition = typesArrayTypeScript.join("\n");
        }
    }
    catch (error) {
        console.log(error.message);
        typeDefinition = null;
    }
    return typeDefinition;
}
