"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = sqlGenOperatorGen;
const sql_equality_parser_1 = __importDefault(require("../../../utils/sql-equality-parser"));
/**
 * # SQL Gen Operator Gen
 * @description Generates an SQL operator for node module `mysql` or `serverless-mysql`
 */
function sqlGenOperatorGen({ fieldName, value, equality, }) {
    if (value) {
        if (equality == "LIKE") {
            return `LOWER(${fieldName}) LIKE LOWER('%${value}%')`;
        }
        else if (equality == "LIKE_RAW") {
            return `LOWER(${fieldName}) LIKE LOWER('${value}')`;
        }
        else if (equality == "NOT LIKE") {
            return `LOWER(${fieldName}) NOT LIKE LOWER('%${value}%')`;
        }
        else if (equality == "NOT LIKE_RAW") {
            return `LOWER(${fieldName}) NOT LIKE LOWER('${value}')`;
        }
        else if (equality == "REGEXP") {
            return `LOWER(${fieldName}) REGEXP LOWER('${value}')`;
        }
        else if (equality == "FULLTEXT") {
            return `MATCH(${fieldName}) AGAINST('${value}' IN BOOLEAN MODE)`;
        }
        else if (equality == "NOT EQUAL") {
            return `${fieldName} != ${value}`;
        }
        else if (equality) {
            return `${fieldName} ${(0, sql_equality_parser_1.default)(equality)} ${value}`;
        }
        else {
            return `${fieldName} = ${value}`;
        }
    }
    else {
        if (equality == "IS NULL") {
            return `${fieldName} IS NULL`;
        }
        else if (equality == "IS NOT NULL") {
            return `${fieldName} IS NOT NULL`;
        }
        else if (equality) {
            return `${fieldName} ${(0, sql_equality_parser_1.default)(equality)} ?`;
        }
        else {
            return `${fieldName} = ?`;
        }
    }
}
