"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
require("dotenv").config({ path: "./../.env" });
const dbHandler_1 = __importDefault(require("../functions/backend/dbHandler"));
const userId = process.argv.indexOf("--userId") >= 0
    ? process.argv[process.argv.indexOf("--userId") + 1]
    : null;
/**
 * Grab Schema
 *
 * @description Grab Schema
 */
function recoverMainJsonFromDb() {
    return __awaiter(this, void 0, void 0, function* () {
        if (!userId) {
            console.log("No user Id provided");
            return;
        }
        const databases = (yield (0, dbHandler_1.default)({
            query: `SELECT * FROM user_databases WHERE user_id='${userId}'`,
        }));
        const dbWrite = [];
        for (let i = 0; i < databases.length; i++) {
            const { id, db_name, db_slug, db_full_name, db_image, db_description } = databases[i];
            const dbObject = {
                dbName: db_name,
                dbSlug: db_slug,
                dbFullName: db_full_name,
                dbDescription: db_description,
                dbImage: db_image,
                tables: [],
            };
            const tables = (yield (0, dbHandler_1.default)({
                query: `SELECT * FROM user_database_tables WHERE user_id='${userId}' AND db_id='${id}'`,
            }));
            for (let j = 0; j < tables.length; j++) {
                const { table_name, table_slug, table_description } = tables[j];
                const tableObject = {
                    tableName: table_slug,
                    tableFullName: table_name,
                    fields: [],
                    indexes: [],
                };
                const tableFields = (yield (0, dbHandler_1.default)({
                    database: db_full_name,
                    query: `SHOW COLUMNS FROM ${db_full_name}.${table_slug}`,
                }));
                if (!tableFields) {
                    console.log("No table fields found");
                    continue;
                }
                for (let k = 0; k < tableFields.length; k++) {
                    const { Field, Type, Null, Default, Key } = tableFields[k];
                    const fieldObject = {
                        fieldName: Field,
                        dataType: Type.toUpperCase(),
                    };
                    if ((Default === null || Default === void 0 ? void 0 : Default.match(/./)) && !(Default === null || Default === void 0 ? void 0 : Default.match(/timestamp/i)))
                        fieldObject["defaultValue"] = Default;
                    if (Key === null || Key === void 0 ? void 0 : Key.match(/pri/i)) {
                        fieldObject["primaryKey"] = true;
                        fieldObject["autoIncrement"] = true;
                    }
                    if (Default === null || Default === void 0 ? void 0 : Default.match(/timestamp/i))
                        fieldObject["defaultValueLiteral"] = Default;
                    if (Null === null || Null === void 0 ? void 0 : Null.match(/yes/i))
                        fieldObject["nullValue"] = true;
                    if (Null === null || Null === void 0 ? void 0 : Null.match(/no/i))
                        fieldObject["notNullValue"] = true;
                    tableObject.fields.push(fieldObject);
                }
                dbObject.tables.push(tableObject);
            }
            dbWrite.push(dbObject);
        }
        fs_1.default.writeFileSync(`${String(process.env.DSQL_USER_DB_SCHEMA_PATH)}/user-${userId}/main.json`, JSON.stringify(dbWrite, null, 4), "utf-8");
        process.exit();
    });
}
recoverMainJsonFromDb();
