"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("dotenv").config({ path: "./../.env" });
const generate_password_1 = __importDefault(require("generate-password"));
const dbHandler_1 = __importDefault(require("../functions/backend/dbHandler"));
const encrypt_1 = __importDefault(require("../functions/dsql/encrypt"));
const grab_sql_key_name_1 = __importDefault(require("../utils/grab-sql-key-name"));
/**
 * # Create database from Schema Function
 */
function resetSQLCredentialsPasswords() {
    return __awaiter(this, void 0, void 0, function* () {
        const users = (yield (0, dbHandler_1.default)({
            query: `SELECT * FROM users`,
        }));
        if (!users) {
            process.exit();
        }
        for (let i = 0; i < users.length; i++) {
            const user = users[i];
            if (!user)
                continue;
            const defaultMariadbUserHost = process.env.DSQL_DB_HOST || "127.0.0.1";
            try {
                const username = (0, grab_sql_key_name_1.default)({ type: "user", userId: user.id });
                const password = generate_password_1.default.generate({
                    length: 16,
                    numbers: true,
                    symbols: true,
                    uppercase: true,
                    exclude: "*#.'`\"",
                });
                const encryptedPassword = (0, encrypt_1.default)({ data: password });
                yield (0, dbHandler_1.default)({
                    query: `SET PASSWORD FOR '${username}'@'${defaultMariadbUserHost}' = PASSWORD('${password}')`,
                });
                const updateUser = yield (0, dbHandler_1.default)({
                    query: `UPDATE users SET mariadb_pass = ? WHERE id = ?`,
                    values: [encryptedPassword, user.id],
                });
                console.log(`User ${user.id}: ${user.first_name} ${user.last_name} Password Updated successfully added.`);
            }
            catch (error) {
                console.log("Error =\u003e", error.message);
            }
        }
        process.exit();
    });
}
resetSQLCredentialsPasswords();
