import connDbHandler from "../../utils/db/conn-db-handler";
import parseDbResults from "./parseDbResults";
import serverError from "./serverError";

type Param = {
    queryString: string;
    tableSchema?: import("../../types").DSQL_TableSchemaType | null;
    queryValuesArray?: string[];
};

/**
 * # Full Access Db Handler
 */
export default async function fullAccessDbHandler({
    queryString,
    tableSchema,
    queryValuesArray,
}: Param) {
    /**
     * Declare variables
     *
     * @description Declare "results" variable
     */
    let results;

    /**
     * Fetch from db
     *
     * @description Fetch data from db if no cache
     */
    try {
        results = await connDbHandler(null, queryString, queryValuesArray);

        ////////////////////////////////////////
    } catch (error: any) {
        ////////////////////////////////////////

        console.log("Full Access DB Handler Error =>", error.message);

        serverError({
            component: "fullAccessDbHandler",
            message: error.message,
        });

        /**
         * Return error
         */
        return error.message;
    } finally {
    }

    /**
     * Return results
     *
     * @description Return results add to cache if "req" param is passed
     */
    if (results && tableSchema) {
        const unparsedResults = results;
        const parsedResults = await parseDbResults({
            unparsedResults: unparsedResults,
            tableSchema: tableSchema,
        });
        return parsedResults;
    } else if (results) {
        return results;
    } else {
        return null;
    }
}
