import { DSQL_DATASQUIREL_MARIADB_USERS } from "../../../types/dsql";
import {
    DatabaseScopedAccessObject,
    UserSQLPermissions,
    UserType,
} from "../../../types";
import grabDbFullName from "../../../utils/grab-db-full-name";
import dbHandler from "../../backend/dbHandler";
import normalizeText from "../../../utils/normalize-text";

type Params = {
    currentAccessedDatabase: DatabaseScopedAccessObject;
    user: UserType;
    updatedRecord: DSQL_DATASQUIREL_MARIADB_USERS;
};

type Return = {
    msg?: string;
    success?: boolean;
};

export default async function handleMariadbUserGrantsForDatabasesRecreateGrants({
    currentAccessedDatabase,
    user,
    updatedRecord,
}: Params): Promise<Return> {
    const { accessedDatabase, dbSlug, allGrants, allTables, grants, tables } =
        currentAccessedDatabase;

    const dbFullName = grabDbFullName({
        user,
        dbName: dbSlug,
    });

    if (allGrants && allTables) {
        const grantAllPrivileges = await dbHandler({
            query: normalizeText(`
                    GRANT ALL PRIVILEGES ON \`${dbFullName}\`.* TO \
                    '${updatedRecord.username}'@'${updatedRecord.host}'
                `),
        });

        return { success: true };
    }

    if (allGrants && tables?.[0]) {
        for (let t = 0; t < tables.length; t++) {
            const table = tables[t];

            // queries.push(
            //     normalizeText(`
            //             GRANT ALL PRIVILEGES ON \`${dbFullName}\`.\`${table.tableSlug}\` \
            //             TO '${updatedRecord.username}'@'${updatedRecord.host}'
            //         `)
            // );

            const grantAllPrivilegesToTables = await dbHandler({
                query: normalizeText(`
                    GRANT ALL PRIVILEGES ON \`${dbFullName}\`.\`${table.tableSlug}\` \
                    TO '${updatedRecord.username}'@'${updatedRecord.host}'
                `),
            });
        }

        return { success: true };
    }

    if (grants?.[0]) {
        const isGrantsInalid = grants.find(
            (g) => !UserSQLPermissions.includes(g)
        );

        if (isGrantsInalid) {
            return { msg: `grants is/are invalid!` };
        }

        if (tables?.[0]) {
            for (let t = 0; t < tables.length; t++) {
                const table = tables[t];

                const grantSpecificPrivilegesToTables = await dbHandler({
                    query: normalizeText(`
                        GRANT ${grants.join(",")} ON \
                        \`${dbFullName}\`.\`${table.tableSlug}\` TO \
                        '${updatedRecord.username}'@'${updatedRecord.host}'
                    `),
                });
            }

            return { success: true };
        } else {
            const grantSecificPrivilegesToAllTables = await dbHandler({
                query: normalizeText(`
                    GRANT ${grants.join(",")} ON \
                    \`${dbFullName}\`.* TO \
                    '${updatedRecord.username}'@'${updatedRecord.host}'
                    `),
            });

            return { success: true };
        }
    }

    return { success: true };
}
