import DataTypes from "../../../data/data-types";
import numberfy from "../../numberfy";

export const DataTypesWithNumbers: (typeof DataTypes)[number]["name"][] = [
    "DECIMAL",
    "DOUBLE",
    "FLOAT",
    "VARCHAR",
];

export const DataTypesWithTwoNumbers: (typeof DataTypes)[number]["name"][] = [
    "DECIMAL",
    "DOUBLE",
    "FLOAT",
];

type Return = {
    type: (typeof DataTypes)[number]["name"];
    limit?: number;
    decimal?: number;
};

export default function dataTypeParser(dataType?: string): Return {
    if (!dataType) {
        return {
            type: "VARCHAR",
            limit: 250,
        };
    }

    const dataTypeArray = dataType.split("(");
    const type = dataTypeArray[0] as (typeof DataTypes)[number]["name"];
    const number = dataTypeArray[1] as string | undefined;

    if (!DataTypesWithNumbers.includes(type)) {
        return {
            type,
        };
    }

    if (number?.match(/,/)) {
        const numberArr = number.split(",");
        return {
            type,
            limit: numberfy(numberArr[0]),
            decimal: numberArr[1] ? numberfy(numberArr[1]) : undefined,
        };
    }

    return {
        type,
        limit: number ? numberfy(number) : undefined,
    };
}
